import { component, useMemo, useLayoutEffect, __useCursor as useCursor } from '@dark-engine/core';
import { illegal, removeContent } from '../utils';
const $$portal = Symbol('portal');
function createPortal(slot, container, key) {
  if (process.env.NODE_ENV !== 'production') {
    if (!(container instanceof Element)) {
      illegal(`The createPortal only gets a valid element as container!`);
    }
  }
  return Portal({ key, container, slot });
}
const Portal = component(
  props => {
    const cursor = useCursor();
    const element = props.container;
    const scope = useMemo(() => {
      removeContent(element);
      return { element };
    }, []);
    useLayoutEffect(() => {
      return () => removeContent(scope.element);
    }, []);
    cursor.hook.setIsPortal(true);
    cursor.element = element;
    scope.element = element;
    return props.slot;
  },
  { token: $$portal, displayName: 'Portal' },
);
export { createPortal };
//# sourceMappingURL=portal.js.map
